VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
END
Attribute VB_Name = "Board"
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit
Const board0x88Height = 16
Const board0x88Width = 16

Private m_arrSquare(board0x88Height * board0x88Width) As Square
Private m_playerWhite As Player
Private m_playerBlack As Player
Private m_playerToPlay As Player
Private m_turnNo As Integer
Private m_turnNoMax As Integer
Private m_movesHistory As Collection
Private m_movesRedoList As Collection

Private m_IsGameOver As Boolean
Private m_resultGame As Integer
Private m_strFENStartPosition As String
Private m_moveNull As Move

Private Sub Class_Initialize()
Dim ordSqr As Byte
Dim sqr As Square
    
    For ordSqr = a1 To h8
        If (ordSqr - a1) And &H88 Then
            Set m_arrSquare(ordSqr) = Nothing
        Else
            Set sqr = New Square
            sqr.Ordinal = ordSqr
            Set m_arrSquare(ordSqr) = sqr
        End If
    Next
    
    Set m_playerWhite = New Player
    Set m_playerBlack = New Player
    
    m_playerWhite.SetOpponent m_playerBlack
    m_playerBlack.SetOpponent m_playerWhite
    
    m_playerWhite.BuildPlayer True, Me, 16, 32, _
        pawnWhiteAttackRightOffset, pawnWhiteAttackLeftOffset
    
    m_playerBlack.BuildPlayer False, Me, -16, -32, _
        pawnBlackAttackRightOffset, pawnBlackAttackLeftOffset
    
    Set m_movesHistory = Nothing
    Set m_movesRedoList = New Collection
    Set m_playerToPlay = m_playerWhite
    
    Set m_moveNull = New Move
    MoveHistoryInit
    m_resultGame = typeUnknown
    
End Sub

Public Sub FreeBoard()
Dim ordSqr As Byte
    
    If Not m_movesHistory Is Nothing Then
        CollMoveClear m_movesHistory
        Set m_movesHistory = Nothing
    End If
    If Not m_movesRedoList Is Nothing Then
        CollMoveClear m_movesRedoList
        Set m_movesRedoList = Nothing
    End If
    Set m_moveNull = Nothing
    
    If Not m_playerBlack Is Nothing Then
        m_playerBlack.FreePlayer
        Set m_playerBlack = Nothing
    End If
    If Not m_playerWhite Is Nothing Then
        m_playerWhite.FreePlayer
        Set m_playerWhite = Nothing
    End If
    
    For ordSqr = a1 To h8
        If Not m_arrSquare(ordSqr) Is Nothing Then
            Set m_arrSquare(ordSqr) = Nothing
        End If
    Next
End Sub

Public Property Get MoveNull() As Move
    Set MoveNull = m_moveNull
End Property

Private Sub CollMoveClear(ByRef colObj As Collection)
Dim moveThis As Move

    While colObj.Count > 0
        Set moveThis = colObj(1)
        colObj.Remove 1
        Set moveThis = Nothing
    Wend
End Sub

Public Property Get IsGameOver() As Boolean
    IsGameOver = m_IsGameOver
End Property

Public Property Let IsGameOver(ByVal Value As Boolean)
    m_IsGameOver = Value
End Property

Public Property Get ResultGame() As Integer
    ResultGame = m_resultGame
End Property

Public Property Let ResultGame(ByVal Value As Integer)
    m_resultGame = Value
End Property

Public Sub GameNew(Optional ByVal strFen As String = strGameStartPosition)

    UndoAllMoves_Internal
    CollMoveClear m_movesRedoList
    m_IsGameOver = False
    m_turnNoMax = 0
    
    strFen = Trim(strFen)
    If Len(strFen) = 0 Then
        strFen = strGameStartPosition
    End If
    SetBoardPosition strFen, Me
    MoveHistoryInit
    m_resultGame = typeUnknown

End Sub

Public Function MakeAMove_Internal(moveThis As Move) As Boolean

    CollMoveClear m_movesRedoList
    moveThis.DoMove
    m_turnNoMax = m_turnNo
    Set m_playerToPlay = m_playerToPlay.OtherPlayer
    moveThis.EnemyStatus = m_playerToPlay.StatusLastMove(moveThis.IsEnemyInCheck)
    
    '
    moveThis.PgnSanFormat True
    If IsStatusOver(moveThis.EnemyStatus) Then
        MakeAMove_Internal = True
    Else
        MakeAMove_Internal = False
    End If

End Function

Public Sub UndoMove_Internal()
Dim moveUndoThis As Move

    Set moveUndoThis = MoveHistoryLast
    If m_movesRedoList.Count = 0 Then
        m_movesRedoList.Add moveUndoThis
    Else
        m_movesRedoList.Add moveUndoThis, , 1
    End If
    moveUndoThis.UndoMove
    Set m_playerToPlay = m_playerToPlay.OtherPlayer

End Sub

Public Function RedoMove_Internal() As Boolean
Dim moveRedoThis As Move, moveLastThis As Move

    Set moveRedoThis = m_movesRedoList(1)
    If Not moveRedoThis Is Nothing Then
        moveRedoThis.DoMove
        Set moveLastThis = MoveHistoryLast
        Set m_playerToPlay = m_playerToPlay.OtherPlayer
        moveLastThis.EnemyStatus = m_playerToPlay.StatusLastMove(moveLastThis.IsEnemyInCheck)
        moveLastThis.Description = moveRedoThis.Description
        m_movesRedoList.Remove 1
        If IsStatusOver(moveLastThis.EnemyStatus) Then
            RedoMove_Internal = True
            Exit Function
        End If
    End If
    RedoMove_Internal = False
End Function

Private Function IsStatusOver(ByVal statusPlayer As Integer) As Boolean

    If statusPlayer > statusDrawStaleMate Then
        IsStatusOver = True
    Else
        IsStatusOver = False
    End If
End Function

Public Sub UndoAllMoves_Internal()

    While m_movesHistory.Count > 1
        UndoMove_Internal
    Wend
End Sub

Public Sub RedoAllMoves_Internal()

    While m_movesRedoList.Count > 0
        RedoMove_Internal
    Wend
End Sub

Public Function MoveHistoryInit()

    If Not m_movesHistory Is Nothing Then
        CollMoveClear m_movesHistory
        Set m_movesHistory = Nothing
    End If
    Set m_movesHistory = New Collection
    m_movesHistory.Add m_moveNull
    
End Function

Public Function GetSquare(ByVal ordSqr As Byte) As Square
    Set GetSquare = m_arrSquare(ordSqr)
End Function

Public Function GetSquareFileRank(ByVal fileSqr As Byte, rankSqr As Byte) As Square
    Set GetSquareFileRank = GetSquare(OrdinalFromFileRank(fileSqr, rankSqr))
End Function

Public Function GetSquareSan(ByVal strSan As String) As Square
Dim indFile As Byte, indRank As Byte

    If Len(strSan) = 2 Then
        indFile = FileFromName(Left(strSan, 1))
        If indFile > 0 Then
            indRank = RankFromName(Right(strSan, 1))
            If indRank > 0 Then
                Set GetSquareSan = GetSquareFileRank(indFile, indRank)
                Exit Function
            End If
        End If
    End If
    Set GetSquareSan = Nothing
End Function

Public Function IsSquareEmpty(ByVal ordSqr As Byte) As Boolean
Dim sqr As Square

    Set sqr = m_arrSquare(ordSqr)
    If sqr Is Nothing Then
        IsSquareEmpty = True
    Else
        IsSquareEmpty = sqr.IsEmpty()
    End If
End Function

Public Function IsRayEmpty(ByVal ordinalStart As Byte, ByVal ordinalEnd As Byte, _
                            ByVal offsetDirection As Integer) As Boolean
Dim ordSqr As Byte, sqr As Square

    ordSqr = ordinalStart + offsetDirection
    While ordSqr <> ordinalEnd
        Set sqr = m_arrSquare(ordSqr)
        If Not sqr.IsEmpty() Then
            IsRayEmpty = False: Exit Function
        End If
        ordSqr = ordSqr + offsetDirection
    Wend
    IsRayEmpty = True
End Function

Public Function IsXRayEmpty(ByVal ordinalStart As Byte, ByVal ordinalEnd As Byte, _
                            ByVal offsetDirection As Integer, ByVal typeSlider As Byte, _
                            ByVal isWhite As Boolean) As Boolean
Dim ordSqr As Byte, pieceThis As Piece, typeOfPiece As Byte, nbrInterceptor As Byte

    ordSqr = ordinalStart + offsetDirection
    nbrInterceptor = 0
    While ordSqr <> ordinalEnd
        Set pieceThis = m_arrSquare(ordSqr).PieceOnSquare
        If Not pieceThis Is Nothing Then
            If pieceThis.isWhite = isWhite Then
                IsXRayEmpty = False: Exit Function
            Else
                typeOfPiece = pieceThis.TypePiece
                If typeOfPiece <> typeQueen Then
                    If typeOfPiece <> typeSlider Then
                        IsXRayEmpty = False: Exit Function
                    End If
                End If
                nbrInterceptor = nbrInterceptor + 1
                If nbrInterceptor > 1 Then
                    IsXRayEmpty = False: Exit Function
                End If
            End If
        End If
        ordSqr = ordSqr + offsetDirection
    Wend
    IsXRayEmpty = True
End Function

Public Function GetPiece(ByVal ordSqr As Byte) As Piece
Dim sqr As Square

    Set sqr = m_arrSquare(ordSqr)
    If sqr Is Nothing Then
        Set GetPiece = Nothing
    Else
        Set GetPiece = sqr.PieceOnSquare
    End If
End Function

Public Function GetPawn(ByVal ordSqr As Byte) As Piece
Dim pieceThis As Piece

    Set pieceThis = GetPiece(ordSqr)
    If pieceThis Is Nothing Then
        Set GetPawn = Nothing
    ElseIf pieceThis.TypePiece <> typePawn Then
        Set GetPawn = Nothing
    Else
        Set GetPawn = pieceThis
    End If
End Function

Public Function GetRook(ByVal ordSqr As Byte, ByVal isWhite As Boolean) As Piece
Dim pieceThis As Piece

    Set pieceThis = GetPiece(ordSqr)
    If pieceThis Is Nothing Then
        Set GetRook = Nothing
    ElseIf pieceThis.TypePiece <> typeRook Then
        Set GetRook = Nothing
    ElseIf pieceThis.isWhite <> isWhite Then
        Set GetRook = Nothing
    Else
        Set GetRook = pieceThis
    End If
End Function

Public Property Get playerToPlay() As Player
    Set playerToPlay = m_playerToPlay
End Property

Public Sub SetPlayerToPlay(ByVal currentPlayer As Player)
    Set m_playerToPlay = currentPlayer
End Sub

Public Property Get playerWhite() As Player
    Set playerWhite = m_playerWhite
End Property

Public Property Get playerBlack() As Player
    Set playerBlack = m_playerBlack
End Property

Public Property Let TurnNo(ByVal Value As Integer)
    m_turnNo = Value
End Property

Public Property Get TurnNo() As Integer
    TurnNo = m_turnNo
End Property

Public Property Get TurnNoMax() As Integer
    TurnNoMax = m_turnNoMax
End Property

Public Function TurnNoIncr() As Integer
    m_turnNo = m_turnNo + 1
    TurnNoIncr = m_turnNo
End Function

Public Function TurnNoDecr() As Integer
    m_turnNo = m_turnNo - 1
    TurnNoDecr = m_turnNo
End Function

Public Property Get IndMoveNo() As Integer
    IndMoveNo = (m_turnNo - 1) \ 2 + 1
End Property

Public Property Let FENStartPosition(ByVal Value As String)
    m_strFENStartPosition = Value
End Property

Public Property Get FENStartPosition() As String
    FENStartPosition = m_strFENStartPosition
End Property

Public Property Get MoveHistory() As Collection
    Set MoveHistory = m_movesHistory
End Property

Public Function MoveHistoryLast() As Move
    Set MoveHistoryLast = m_movesHistory(m_movesHistory.Count)
End Function

Public Property Get MovesRedoList() As Collection
    Set MovesRedoList = m_movesRedoList
End Property

Public Sub MoveHistoryRemoveLast()
    m_movesHistory.Remove m_movesHistory.Count
End Sub

Public Function GoToHalfMove(ByVal turnNoWished As Integer) As Boolean
Dim isTurnNoChanged As Boolean

    isTurnNoChanged = False
    If turnNoWished < 0 Then ErrWarn 8000
    If turnNoWished <= m_turnNoMax Then
        While m_turnNo > turnNoWished
            UndoMove_Internal
            isTurnNoChanged = True
        Wend
        While m_turnNo < turnNoWished
            RedoMove_Internal
            isTurnNoChanged = True
        Wend
    End If
    GoToHalfMove = isTurnNoChanged
    
End Function

Public Function IsEndGame() As Boolean
Const valueEndGame = valueRook * 2
Dim isEndOfGame As Boolean

    isEndOfGame = False
    If m_playerWhite.SumValuePiece() <= valueEndGame Then
        isEndOfGame = True
    ElseIf m_playerBlack.SumValuePiece() <= valueEndGame Then
        isEndOfGame = True
    End If
    IsEndGame = isEndOfGame
    
End Function






